from _typeshed import Incomplete

ZSTD_MAX_OUTPUT_SIZE: Incomplete
PYPY: Incomplete

def has_gzip(): ...
def has_snappy(): ...
def has_zstd(): ...
def has_lz4(): ...
def gzip_encode(payload, compresslevel=None): ...
def gzip_decode(payload): ...
def snappy_encode(payload, xerial_compatible: bool = True, xerial_blocksize=32768): ...
def snappy_decode(payload): ...

lz4_encode: Incomplete

def lz4f_decode(payload): ...

lz4_decode: Incomplete
lz4_decode = lz4f_decode

def lz4_encode_old_kafka(payload): ...
def lz4_decode_old_kafka(payload): ...
def zstd_encode(payload): ...
def zstd_decode(payload): ...



from _typeshed import Incomplete
from typing import NamedTuple

from kafka.coordinator.assignors.abstract import AbstractPartitionAssignor
from kafka.protocol.struct import Struct

log: Incomplete

class ConsumerGenerationPair(NamedTuple):
    consumer: Incomplete
    generation: Incomplete

def has_identical_list_elements(list_): ...
def subscriptions_comparator_key(element): ...
def partitions_comparator_key(element): ...
def remove_if_present(collection, element) -> None: ...

class StickyAssignorMemberMetadataV1(NamedTuple):
    subscription: Incomplete
    partitions: Incomplete
    generation: Incomplete

class StickyAssignorUserDataV1(Struct):
    SCHEMA: Incomplete

class StickyAssignmentExecutor:
    members: Incomplete
    current_assignment: Incomplete
    previous_assignment: Incomplete
    current_partition_consumer: Incomplete
    is_fresh_assignment: bool
    partition_to_all_potential_consumers: Incomplete
    consumer_to_all_potential_partitions: Incomplete
    sorted_current_subscriptions: Incomplete
    sorted_partitions: Incomplete
    unassigned_partitions: Incomplete
    revocation_required: bool
    partition_movements: Incomplete
    def __init__(self, cluster, members) -> None: ...
    def perform_initial_assignment(self) -> None: ...
    def balance(self) -> None: ...
    def get_final_assignment(self, member_id): ...

class StickyPartitionAssignor(AbstractPartitionAssignor):
    DEFAULT_GENERATION_ID: int
    name: str
    version: int
    member_assignment: Incomplete
    generation = DEFAULT_GENERATION_ID
    @classmethod
    def assign(cls, cluster, members): ...
    @classmethod
    def parse_member_metadata(cls, metadata): ...
    @classmethod
    def metadata(cls, topics): ...
    @classmethod
    def on_assignment(cls, assignment) -> None: ...
    @classmethod
    def on_generation_assignment(cls, generation) -> None: ...



from _typeshed import Incomplete

class MetricConfig:
    quota: Incomplete
    event_window: Incomplete
    time_window_ms: Incomplete
    tags: Incomplete
    def __init__(self, quota=None, samples: int = 2, event_window=..., time_window_ms=30000, tags=None) -> None: ...

    @property
    def samples(self): ...
    @samples.setter
    def samples(self, value) -> None: ...



from _typeshed import Incomplete
from typing import Final, Literal

__version__: Final[str]

def makeA85Image(filename, IMG=None, detectJpeg: bool = False) -> list[Incomplete] | None: ...
def makeRawImage(filename, IMG=None, detectJpeg: bool = False) -> list[Incomplete] | None: ...
def cacheImageFile(filename, returnInMemory: bool | Literal[0, 1] = 0, IMG=None): ...
def preProcessImages(spec) -> None: ...
def cachedImageExists(filename) -> Literal[0, 1]: ...
def readJPEGInfo(image) -> tuple[Incomplete, Incomplete, Incomplete, Incomplete] | None: ...

class _fusc:
    def __init__(self, k, n) -> None: ...
    def encrypt(self, s) -> str: ...
    def decrypt(self, s) -> str: ...



from _typeshed import SupportsWrite
from collections.abc import Callable, Iterable, Mapping
from typing import Any, TypedDict, TypeVar, overload, type_check_only
from typing_extensions import Unpack

from ..std import tqdm

__all__ = ["thread_map", "process_map"]

_R = TypeVar("_R")
_T1 = TypeVar("_T1")
_T2 = TypeVar("_T2")
_T3 = TypeVar("_T3")
_T4 = TypeVar("_T4")
_T5 = TypeVar("_T5")

@type_check_only
class _TqdmKwargs(TypedDict, total=False):
    # Concurrent-specific parameters
    tqdm_class: type[tqdm[object]]
    max_workers: int | None
    chunksize: int
    lock_name: str
    # Standard tqdm parameters
    desc: str | None
    total: float | None
    leave: bool | None
    file: SupportsWrite[str] | None
    ncols: int | None
    mininterval: float
    maxinterval: float
    miniters: float | None
    ascii: bool | str | None
    disable: bool | None
    unit: str
    unit_scale: bool | float
    dynamic_ncols: bool
    smoothing: float
    bar_format: str | None
    initial: float
    position: int | None
    postfix: Mapping[str, object] | str | None
    unit_divisor: float
    write_bytes: bool | None
    lock_args: tuple[bool | None, float | None] | tuple[bool | None] | None
    nrows: int | None
    colour: str | None
    delay: float | None

@overload
def thread_map(fn: Callable[[_T1], _R], iter1: Iterable[_T1], **tqdm_kwargs: Unpack[_TqdmKwargs]) -> list[_R]: ...
@overload
def thread_map(
    fn: Callable[[_T1, _T2], _R], iter1: Iterable[_T1], iter2: Iterable[_T2], /, **tqdm_kwargs: Unpack[_TqdmKwargs]
) -> list[_R]: ...
@overload
def thread_map(
    fn: Callable[[_T1, _T2, _T3], _R],
    iter1: Iterable[_T1],
    iter2: Iterable[_T2],
    iter3: Iterable[_T3],
    **tqdm_kwargs: Unpack[_TqdmKwargs],
) -> list[_R]: ...
@overload
def thread_map(
    fn: Callable[[_T1, _T2, _T3, _T4], _R],
    iter1: Iterable[_T1],
    iter2: Iterable[_T2],
    iter3: Iterable[_T3],
    iter4: Iterable[_T4],
    **tqdm_kwargs: Unpack[_TqdmKwargs],
) -> list[_R]: ...
@overload
def thread_map(
    fn: Callable[[_T1, _T2, _T3, _T4, _T5], _R],
    iter1: Iterable[_T1],
    iter2: Iterable[_T2],
    iter3: Iterable[_T3],
    iter4: Iterable[_T4],
    iter5: Iterable[_T5],
    **tqdm_kwargs: Unpack[_TqdmKwargs],
) -> list[_R]: ...
@overload
def thread_map(
    fn: Callable[..., _R],
    iter1: Iterable[Any],
    iter2: Iterable[Any],
    iter3: Iterable[Any],
    iter4: Iterable[Any],
    iter5: Iterable[Any],
    iter6: Iterable[Any],
    *iterables: Iterable[Any],
    **tqdm_kwargs: Unpack[_TqdmKwargs],
) -> list[_R]: ...

@overload
def process_map(fn: Callable[[_T1], _R], iter1: Iterable[_T1], **tqdm_kwargs: Unpack[_TqdmKwargs]) -> list[_R]: ...
@overload
def process_map(
    fn: Callable[[_T1, _T2], _R], iter1: Iterable[_T1], iter2: Iterable[_T2], **tqdm_kwargs: Unpack[_TqdmKwargs]
) -> list[_R]: ...
@overload
def process_map(
    fn: Callable[[_T1, _T2, _T3], _R],
    iter1: Iterable[_T1],
    iter2: Iterable[_T2],
    iter3: Iterable[_T3],
    **tqdm_kwargs: Unpack[_TqdmKwargs],
) -> list[_R]: ...
@overload
def process_map(
    fn: Callable[[_T1, _T2, _T3, _T4], _R],
    iter1: Iterable[_T1],
    iter2: Iterable[_T2],
    iter3: Iterable[_T3],
    iter4: Iterable[_T4],
    **tqdm_kwargs: Unpack[_TqdmKwargs],
) -> list[_R]: ...
@overload
def process_map(
    fn: Callable[[_T1, _T2, _T3, _T4, _T5], _R],
    iter1: Iterable[_T1],
    iter2: Iterable[_T2],
    iter3: Iterable[_T3],
    iter4: Iterable[_T4],
    iter5: Iterable[_T5],
    **tqdm_kwargs: Unpack[_TqdmKwargs],
) -> list[_R]: ...
@overload
def process_map(
    fn: Callable[..., _R],
    iter1: Iterable[Any],
    iter2: Iterable[Any],
    iter3: Iterable[Any],
    iter4: Iterable[Any],
    iter5: Iterable[Any],
    iter6: Iterable[Any],
    *iterables: Iterable[Any],
    **tqdm_kwargs: Unpack[_TqdmKwargs],
) -> list[_R]: ...



from _typeshed import Incomplete

from authlib.oauth2.auth import ClientAuth, TokenAuth
from authlib.oauth2.client import OAuth2Client

from ..base_client import OAuthError

__all__ = ["OAuth2Session", "OAuth2Auth"]

# Inherits from requests.auth.AuthBase
class OAuth2Auth(TokenAuth):
    def ensure_active_token(self) -> None: ...
    def __call__(self, req): ...

# Inherits from requests.auth.AuthBase
class OAuth2ClientAuth(ClientAuth):
    def __call__(self, req): ...

# Inherits from requests.Session
class OAuth2Session(OAuth2Client):
    client_auth_class = OAuth2ClientAuth
    token_auth_class = OAuth2Auth
    oauth_error_class = OAuthError  # type: ignore[assignment]
    SESSION_REQUEST_PARAMS: tuple[str, ...]  # type: ignore[assignment]
    default_timeout: Incomplete
    def __init__(
        self,
        client_id=None,
        client_secret=None,
        token_endpoint_auth_method=None,
        revocation_endpoint_auth_method=None,
        scope=None,
        state=None,
        redirect_uri=None,
        token=None,
        token_placement="header",
        update_token=None,
        leeway=60,
        default_timeout=None,
        **kwargs,
    ) -> None: ...
    def fetch_access_token(self, url=None, **kwargs): ...
    def request(self, method, url, withhold_token=False, auth=None, **kwargs): ...



from _typeshed import Incomplete

from kafka.protocol.abstract import AbstractType as AbstractType

class Int8(AbstractType):
    @classmethod
    def encode(cls, value): ...
    @classmethod
    def decode(cls, data): ...

class Int16(AbstractType):
    @classmethod
    def encode(cls, value): ...
    @classmethod
    def decode(cls, data): ...

class Int32(AbstractType):
    @classmethod
    def encode(cls, value): ...
    @classmethod
    def decode(cls, data): ...

class Int64(AbstractType):
    @classmethod
    def encode(cls, value): ...
    @classmethod
    def decode(cls, data): ...

class Float64(AbstractType):
    @classmethod
    def encode(cls, value): ...
    @classmethod
    def decode(cls, data): ...

class String(AbstractType):
    encoding: Incomplete
    def __init__(self, encoding: str = "utf-8") -> None: ...
    def encode(self, value): ...
    def decode(self, data): ...

class Bytes(AbstractType):
    @classmethod
    def encode(cls, value): ...
    @classmethod
    def decode(cls, data): ...
    @classmethod
    def repr(cls, value): ...

class Boolean(AbstractType):
    @classmethod
    def encode(cls, value): ...
    @classmethod
    def decode(cls, data): ...

class Schema(AbstractType):
    def __init__(self, *fields) -> None: ...
    def encode(self, item): ...
    def decode(self, data): ...
    def __len__(self) -> int: ...
    def repr(self, value): ...

class Array(AbstractType):
    array_of: Incomplete
    def __init__(self, *array_of) -> None: ...
    def encode(self, items): ...
    def decode(self, data): ...
    def repr(self, list_of_items): ...

class UnsignedVarInt32(AbstractType):
    @classmethod
    def decode(cls, data): ...
    @classmethod
    def encode(cls, value): ...

class VarInt32(AbstractType):
    @classmethod
    def decode(cls, data): ...
    @classmethod
    def encode(cls, value): ...

class VarInt64(AbstractType):
    @classmethod
    def decode(cls, data): ...
    @classmethod
    def encode(cls, value): ...

class CompactString(String):
    def decode(self, data): ...
    def encode(self, value): ...

class TaggedFields(AbstractType):
    @classmethod
    def decode(cls, data): ...
    @classmethod
    def encode(cls, value): ...

class CompactBytes(AbstractType):
    @classmethod
    def decode(cls, data): ...
    @classmethod
    def encode(cls, value): ...

class CompactArray(Array):
    def encode(self, items): ...
    def decode(self, data): ...

class BitField(AbstractType):
    @classmethod
    def decode(cls, data): ...
    @classmethod
    def encode(cls, vals): ...
    @classmethod
    def to_32_bit_field(cls, vals): ...
    @classmethod
    def from_32_bit_field(cls, value): ...



from _typeshed import Incomplete

from kafka.record.abc import ABCRecord, ABCRecordBatch, ABCRecordBatchBuilder

class DefaultRecordBase:
    HEADER_STRUCT: Incomplete
    ATTRIBUTES_OFFSET: Incomplete
    CRC_OFFSET: Incomplete
    AFTER_LEN_OFFSET: Incomplete
    CODEC_MASK: int
    CODEC_NONE: int
    CODEC_GZIP: int
    CODEC_SNAPPY: int
    CODEC_LZ4: int
    CODEC_ZSTD: int
    TIMESTAMP_TYPE_MASK: int
    TRANSACTIONAL_MASK: int
    CONTROL_MASK: int
    LOG_APPEND_TIME: int
    CREATE_TIME: int
    NO_PRODUCER_ID: int
    NO_SEQUENCE: int
    MAX_INT: int

class DefaultRecordBatch(DefaultRecordBase, ABCRecordBatch):
    def __init__(self, buffer) -> None: ...
    @property
    def base_offset(self): ...
    @property
    def size_in_bytes(self): ...
    @property
    def leader_epoch(self): ...
    @property
    def magic(self): ...
    @property
    def crc(self): ...
    @property
    def attributes(self): ...
    @property
    def last_offset_delta(self): ...
    @property
    def last_offset(self): ...
    @property
    def next_offset(self): ...
    @property
    def compression_type(self): ...
    @property
    def timestamp_type(self): ...
    @property
    def is_transactional(self): ...
    @property
    def is_control_batch(self): ...
    @property
    def first_timestamp(self): ...
    @property
    def max_timestamp(self): ...
    @property
    def producer_id(self): ...
    def has_producer_id(self): ...
    @property
    def producer_epoch(self): ...
    @property
    def base_sequence(self): ...
    @property
    def has_sequence(self): ...
    @property
    def last_sequence(self): ...
    @property
    def records_count(self): ...
    def __iter__(self): ...
    def __next__(self): ...
    next = __next__
    def validate_crc(self): ...

class DefaultRecord(ABCRecord):
    def __init__(self, size_in_bytes, offset, timestamp, timestamp_type, key, value, headers) -> None: ...
    @property
    def size_in_bytes(self): ...
    @property
    def offset(self): ...
    @property
    def timestamp(self): ...
    @property
    def timestamp_type(self): ...
    @property
    def key(self): ...
    @property
    def value(self): ...
    @property
    def headers(self): ...
    @property
    def checksum(self) -> None: ...
    def validate_crc(self): ...

class ControlRecord(DefaultRecord):
    KEY_STRUCT: Incomplete
    def __init__(self, size_in_bytes, offset, timestamp, timestamp_type, key, value, headers) -> None: ...
    @property
    def version(self): ...
    @property
    def type(self): ...
    @property
    def abort(self): ...
    @property
    def commit(self): ...

class DefaultRecordBatchBuilder(DefaultRecordBase, ABCRecordBatchBuilder):
    MAX_RECORD_OVERHEAD: int
    def __init__(
        self, magic, compression_type, is_transactional, producer_id, producer_epoch, base_sequence, batch_size
    ) -> None: ...
    def set_producer_state(self, producer_id, producer_epoch, base_sequence, is_transactional) -> None: ...
    @property
    def producer_id(self): ...
    @property
    def producer_epoch(self): ...
    def append(  # type: ignore[override]
        self,
        offset,
        timestamp,
        key,
        value,
        headers,
        encode_varint=...,
        size_of_varint=...,
        get_type=...,
        type_int=...,
        time_time=...,
        byte_like=...,
        bytearray_type=...,
        len_func=...,
        zero_len_varint: int = 1,
    ): ...
    def write_header(self, use_compression_type: bool = True) -> None: ...
    def build(self): ...
    def size(self): ...
    @classmethod
    def header_size_in_bytes(cls): ...
    @classmethod
    def size_in_bytes(cls, offset_delta, timestamp_delta, key, value, headers): ...
    @classmethod
    def size_of(cls, key, value, headers): ...
    @classmethod
    def estimate_size_in_bytes(cls, key, value, headers): ...

class DefaultRecordMetadata:
    def __init__(self, offset, size, timestamp) -> None: ...
    @property
    def offset(self): ...
    @property
    def crc(self) -> None: ...
    @property
    def size(self): ...
    @property
    def timestamp(self): ...



from _typeshed import Incomplete
from collections.abc import Iterable
from typing import Literal

import _win32typing

def wrap(
    ob, iid=None, usePolicy: type[Incomplete] | None = None, useDispatcher: type[Incomplete] | bool | Literal[0, 1] | None = None
): ...
def unwrap(ob): ...

class ListEnumerator:
    index: int
    def __init__(self, data, index: int = 0, iid=...) -> None: ...
    def Next(self, count: int): ...
    def Skip(self, count: int) -> None: ...
    def Reset(self) -> None: ...
    def Clone(self): ...

class ListEnumeratorGateway(ListEnumerator):
    def Next(self, count: int) -> Iterable[Incomplete]: ...

def NewEnum(
    seq,
    cls=...,
    iid=...,
    usePolicy: type[Incomplete] | None = None,
    useDispatcher: type[Incomplete] | bool | Literal[0, 1] | None = None,
): ...

class Collection:
    data: Incomplete
    def __init__(self, data=None, readOnly: bool | Literal[0, 1] = 0) -> None: ...
    def Item(self, *args): ...
    def Count(self): ...
    def Add(self, value) -> None: ...
    def Remove(self, index) -> None: ...
    def Insert(self, index, value) -> None: ...

def NewCollection(seq, cls=...) -> _win32typing.PyIUnknown: ...

class FileStream:
    file: Incomplete
    def __init__(self, file: _win32typing.Pymmapfile) -> None: ...
    def Read(self, amount): ...
    def Write(self, data) -> int: ...
    def Clone(self): ...
    def CopyTo(self, dest, cb) -> tuple[int, int]: ...
    def Seek(self, offset: int, origin: int) -> int: ...



from _typeshed import Incomplete

def constant_to_class_kind(value): ...
def constant_to_attribute_usage(value): ...
def attribute_usage_to_constant(value): ...
def quoted_string_to_list(quoted_string): ...
def oids_string_to_list(oid_string): ...
def extension_to_tuple(extension_string): ...
def list_to_string(list_object): ...

class BaseServerInfo:
    raw: Incomplete
    def __init__(self, raw_attributes) -> None: ...
    @classmethod
    def from_json(cls, json_definition, schema=None, custom_formatter=None): ...
    @classmethod
    def from_file(cls, target, schema=None, custom_formatter=None): ...
    def to_file(self, target, indent: int = 4, sort: bool = True) -> None: ...
    def to_json(self, indent: int = 4, sort: bool = True): ...

class DsaInfo(BaseServerInfo):
    alt_servers: Incomplete
    naming_contexts: Incomplete
    supported_controls: Incomplete
    supported_extensions: Incomplete
    supported_features: Incomplete
    supported_ldap_versions: Incomplete
    supported_sasl_mechanisms: Incomplete
    vendor_name: Incomplete
    vendor_version: Incomplete
    schema_entry: Incomplete
    other: Incomplete
    def __init__(self, attributes, raw_attributes) -> None: ...

class SchemaInfo(BaseServerInfo):
    schema_entry: Incomplete
    create_time_stamp: Incomplete
    modify_time_stamp: Incomplete
    attribute_types: Incomplete
    object_classes: Incomplete
    matching_rules: Incomplete
    matching_rule_uses: Incomplete
    dit_content_rules: Incomplete
    dit_structure_rules: Incomplete
    name_forms: Incomplete
    ldap_syntaxes: Incomplete
    other: Incomplete
    def __init__(self, schema_entry, attributes, raw_attributes) -> None: ...
    def is_valid(self): ...

class BaseObjectInfo:
    oid: Incomplete
    name: Incomplete
    description: Incomplete
    obsolete: Incomplete
    extensions: Incomplete
    experimental: Incomplete
    raw_definition: Incomplete
    def __init__(
        self, oid=None, name=None, description=None, obsolete: bool = False, extensions=None, experimental=None, definition=None
    ) -> None: ...
    @property
    def oid_info(self): ...
    @classmethod
    def from_definition(cls, definitions): ...

class MatchingRuleInfo(BaseObjectInfo):
    syntax: Incomplete
    def __init__(
        self,
        oid=None,
        name=None,
        description=None,
        obsolete: bool = False,
        syntax=None,
        extensions=None,
        experimental=None,
        definition=None,
    ) -> None: ...

class MatchingRuleUseInfo(BaseObjectInfo):
    apply_to: Incomplete
    def __init__(
        self,
        oid=None,
        name=None,
        description=None,
        obsolete: bool = False,
        apply_to=None,
        extensions=None,
        experimental=None,
        definition=None,
    ) -> None: ...

class ObjectClassInfo(BaseObjectInfo):
    superior: Incomplete
    kind: Incomplete
    must_contain: Incomplete
    may_contain: Incomplete
    def __init__(
        self,
        oid=None,
        name=None,
        description=None,
        obsolete: bool = False,
        superior=None,
        kind=None,
        must_contain=None,
        may_contain=None,
        extensions=None,
        experimental=None,
        definition=None,
    ) -> None: ...

class AttributeTypeInfo(BaseObjectInfo):
    superior: Incomplete
    equality: Incomplete
    ordering: Incomplete
    substring: Incomplete
    syntax: Incomplete
    min_length: Incomplete
    single_value: Incomplete
    collective: Incomplete
    no_user_modification: Incomplete
    usage: Incomplete
    mandatory_in: Incomplete
    optional_in: Incomplete
    def __init__(
        self,
        oid=None,
        name=None,
        description=None,
        obsolete: bool = False,
        superior=None,
        equality=None,
        ordering=None,
        substring=None,
        syntax=None,
        min_length=None,
        single_value: bool = False,
        collective: bool = False,
        no_user_modification: bool = False,
        usage=None,
        extensions=None,
        experimental=None,
        definition=None,
    ) -> None: ...

class LdapSyntaxInfo(BaseObjectInfo):
    def __init__(self, oid=None, description=None, extensions=None, experimental=None, definition=None) -> None: ...

class DitContentRuleInfo(BaseObjectInfo):
    auxiliary_classes: Incomplete
    must_contain: Incomplete
    may_contain: Incomplete
    not_contains: Incomplete
    def __init__(
        self,
        oid=None,
        name=None,
        description=None,
        obsolete: bool = False,
        auxiliary_classes=None,
        must_contain=None,
        may_contain=None,
        not_contains=None,
        extensions=None,
        experimental=None,
        definition=None,
    ) -> None: ...

class DitStructureRuleInfo(BaseObjectInfo):
    superior: Incomplete
    name_form: Incomplete
    def __init__(
        self,
        oid=None,
        name=None,
        description=None,
        obsolete: bool = False,
        name_form=None,
        superior=None,
        extensions=None,
        experimental=None,
        definition=None,
    ) -> None: ...

class NameFormInfo(BaseObjectInfo):
    object_class: Incomplete
    must_contain: Incomplete
    may_contain: Incomplete
    def __init__(
        self,
        oid=None,
        name=None,
        description=None,
        obsolete: bool = False,
        object_class=None,
        must_contain=None,
        may_contain=None,
        extensions=None,
        experimental=None,
        definition=None,
    ) -> None: ...



class Sensor:
    def __init__(self, registry, name, parents, config, inactive_sensor_expiration_time_seconds) -> None: ...
    @property
    def name(self): ...
    @property
    def metrics(self): ...
    def record(self, value: float = 1.0, time_ms=None) -> None: ...
    def add_compound(self, compound_stat, config=None) -> None: ...
    def add(self, metric_name, stat, config=None) -> None: ...
    def has_expired(self): ...



from collections.abc import Mapping
from typing import Any, TypeAlias

from yaml.emitter import Emitter
from yaml.representer import BaseRepresenter, Representer, SafeRepresenter
from yaml.resolver import BaseResolver, Resolver
from yaml.serializer import Serializer

from .emitter import _WriteStream

# Ideally, there would be a way to limit these values to only +/- float("inf"),
# but that's not possible at the moment (https://github.com/python/typing/issues/1160).
_Inf: TypeAlias = float

class BaseDumper(Emitter, Serializer, BaseRepresenter, BaseResolver):
    def __init__(
        self,
        stream: _WriteStream[Any],
        default_style: str | None = None,
        default_flow_style: bool | None = False,
        canonical: bool | None = None,
        indent: int | None = None,
        width: int | _Inf | None = None,
        allow_unicode: bool | None = None,
        line_break: str | None = None,
        encoding: str | None = None,
        explicit_start: bool | None = None,
        explicit_end: bool | None = None,
        version: tuple[int, int] | None = None,
        tags: Mapping[str, str] | None = None,
        sort_keys: bool = True,
    ) -> None: ...

class SafeDumper(Emitter, Serializer, SafeRepresenter, Resolver):
    def __init__(
        self,
        stream: _WriteStream[Any],
        default_style: str | None = None,
        default_flow_style: bool | None = False,
        canonical: bool | None = None,
        indent: int | None = None,
        width: int | _Inf | None = None,
        allow_unicode: bool | None = None,
        line_break: str | None = None,
        encoding: str | None = None,
        explicit_start: bool | None = None,
        explicit_end: bool | None = None,
        version: tuple[int, int] | None = None,
        tags: Mapping[str, str] | None = None,
        sort_keys: bool = True,
    ) -> None: ...

class Dumper(Emitter, Serializer, Representer, Resolver):
    def __init__(
        self,
        stream: _WriteStream[Any],
        default_style: str | None = None,
        default_flow_style: bool | None = False,
        canonical: bool | None = None,
        indent: int | None = None,
        width: int | _Inf | None = None,
        allow_unicode: bool | None = None,
        line_break: str | None = None,
        encoding: str | None = None,
        explicit_start: bool | None = None,
        explicit_end: bool | None = None,
        version: tuple[int, int] | None = None,
        tags: Mapping[str, str] | None = None,
        sort_keys: bool = True,
    ) -> None: ...

__all__ = ["BaseDumper", "SafeDumper", "Dumper"]



from _typeshed import Incomplete
from typing import Any

from ..base_client import FrameworkIntegration

class StarletteIntegration(FrameworkIntegration):
    # annotated by source code
    async def get_state_data(self, session: dict[str, Any] | None, state: str) -> dict[str, Any]: ...
    async def set_state_data(self, session: dict[str, Any] | None, state: str, data: Any) -> None: ...
    async def clear_state_data(self, session: dict[str, Any] | None, state: str) -> None: ...
    def update_token(self, token, refresh_token=None, access_token=None) -> None: ...
    @staticmethod
    def load_config(oauth, name, params) -> dict[Incomplete, Incomplete]: ...



from _io import _BufferedReaderStream
from _typeshed import Incomplete
from socket import SocketIO
from typing import Literal, overload

from docker.transport.sshconn import SSHSocket
from docker.types.daemon import CancellableStream

class ExecApiMixin:
    def exec_create(
        self,
        container,
        cmd,
        stdout: bool = True,
        stderr: bool = True,
        stdin: bool = False,
        tty: bool = False,
        privileged: bool = False,
        user: str = "",
        environment: dict[str, str] | list[str] | None = None,
        workdir: str | None = None,
        detach_keys: str | None = None,
    ) -> dict[str, Incomplete]: ...
    def exec_inspect(self, exec_id: str) -> dict[str, Incomplete]: ...
    def exec_resize(self, exec_id: str, height: int | None = None, width: int | None = None) -> None: ...

    @overload
    def exec_start(
        self,
        exec_id: str,
        detach: Literal[True],
        tty: bool = False,
        stream: bool = False,
        socket: bool = False,
        demux: bool = False,
    ) -> bytes: ...
    @overload
    def exec_start(
        self, exec_id: str, detach: Literal[False], tty: bool, stream: bool, socket: Literal[True], demux: bool = False
    ) -> SocketIO | _BufferedReaderStream | SSHSocket: ...
    @overload
    def exec_start(
        self,
        exec_id: str,
        detach: Literal[False] = False,
        tty: bool = False,
        stream: bool = False,
        *,
        socket: Literal[True],
        demux: bool = False,
    ) -> SocketIO | _BufferedReaderStream | SSHSocket: ...
    @overload
    def exec_start(
        self, exec_id: str, detach: Literal[False], tty: bool, stream: Literal[True], socket: Literal[False], demux: Literal[True]
    ) -> CancellableStream[tuple[bytes | None, bytes | None]]: ...
    @overload
    def exec_start(
        self,
        exec_id: str,
        detach: Literal[False] = False,
        tty: bool = False,
        socket: Literal[False] = False,
        *,
        stream: Literal[True],
        demux: Literal[True],
    ) -> CancellableStream[tuple[bytes | None, bytes | None]]: ...
    @overload
    def exec_start(
        self,
        exec_id: str,
        detach: Literal[False],
        tty: bool,
        stream: Literal[True],
        socket: Literal[False],
        demux: Literal[False],
    ) -> CancellableStream[bytes]: ...
    @overload
    def exec_start(
        self,
        exec_id: str,
        detach: Literal[False] = False,
        tty: bool = False,
        *,
        stream: Literal[True],
        socket: Literal[False] = False,
        demux: Literal[False] = False,
    ) -> CancellableStream[bytes]: ...
    @overload
    def exec_start(
        self,
        exec_id: str,
        detach: Literal[False],
        tty: bool,
        stream: Literal[False],
        socket: Literal[False],
        demux: Literal[True],
    ) -> tuple[bytes | None, bytes | None]: ...
    @overload
    def exec_start(
        self,
        exec_id: str,
        detach: Literal[False] = False,
        tty: bool = False,
        stream: Literal[False] = False,
        socket: Literal[False] = False,
        *,
        demux: Literal[True],
    ) -> tuple[bytes | None, bytes | None]: ...
    @overload
    def exec_start(
        self,
        exec_id: str,
        detach: Literal[False] = False,
        tty: bool = False,
        stream: Literal[False] = False,
        socket: Literal[False] = False,
        demux: Literal[False] = False,
    ) -> bytes: ...
    @overload
    def exec_start(
        self,
        exec_id: str,
        detach: bool = False,
        tty: bool = False,
        stream: bool = False,
        socket: bool = False,
        demux: bool = False,
    ) -> (
        str
        | SocketIO
        | _BufferedReaderStream
        | SSHSocket
        | CancellableStream[bytes]
        | CancellableStream[tuple[bytes | None, bytes | None]]
        | tuple[bytes | None, bytes | None]
        | bytes
    ): ...



from gunicorn.ctl.client import ControlClient as ControlClient
from gunicorn.ctl.protocol import ControlProtocol as ControlProtocol
from gunicorn.ctl.server import ControlSocketServer as ControlSocketServer

__all__ = ["ControlSocketServer", "ControlClient", "ControlProtocol"]



import abc

from kafka.metrics.measurable import AbstractMeasurable
from kafka.metrics.stat import AbstractStat

class AbstractMeasurableStat(AbstractStat, AbstractMeasurable, metaclass=abc.ABCMeta): ...



from kafka.admin.acl_resource import (
    ACL as ACL,
    ACLFilter as ACLFilter,
    ACLOperation as ACLOperation,
    ACLPermissionType as ACLPermissionType,
    ACLResourcePatternType as ACLResourcePatternType,
    ResourcePattern as ResourcePattern,
    ResourcePatternFilter as ResourcePatternFilter,
    ResourceType as ResourceType,
)
from kafka.admin.client import KafkaAdminClient as KafkaAdminClient
from kafka.admin.config_resource import ConfigResource as ConfigResource, ConfigResourceType as ConfigResourceType
from kafka.admin.new_partitions import NewPartitions as NewPartitions
from kafka.admin.new_topic import NewTopic as NewTopic

__all__ = [
    "ConfigResource",
    "ConfigResourceType",
    "KafkaAdminClient",
    "NewTopic",
    "NewPartitions",
    "ACL",
    "ACLFilter",
    "ResourcePattern",
    "ResourcePatternFilter",
    "ACLOperation",
    "ResourceType",
    "ACLPermissionType",
    "ACLResourcePatternType",
]



from collections.abc import Callable, Generator, Iterable
from typing import Any, TypeVar

from ..std import tqdm as std_tqdm

__all__ = ["chain", "product", "permutations", "combinations", "combinations_with_replacement", "batched"]

_T = TypeVar("_T")

# In all functions below, kwargs are forwarded to tqdm_class.
def chain(
    *iterables: Iterable[_T], total: int | None = None, tqdm_class: Callable[..., std_tqdm[_T]] = ..., **kwargs: Any
) -> std_tqdm[_T]: ...
def product(
    *iterables: Iterable[_T],
    repeat: int = 1,
    total: int | None = None,
    tqdm_class: Callable[..., std_tqdm[tuple[_T, ...]]] = ...,
    **kwargs: Any,
) -> Generator[tuple[_T, ...]]: ...
def permutations(
    iterable: Iterable[_T],
    r: int | None = None,
    total: int | None = None,
    tqdm_class: Callable[..., std_tqdm[tuple[_T, ...]]] = ...,
    **kwargs: Any,
) -> std_tqdm[tuple[_T, ...]]: ...
def combinations(
    iterable: Iterable[_T],
    r: int,
    total: int | None = None,
    tqdm_class: Callable[..., std_tqdm[tuple[_T, ...]]] = ...,
    **kwargs: Any,
) -> std_tqdm[tuple[_T, ...]]: ...
def combinations_with_replacement(
    iterable: Iterable[_T],
    r: int,
    total: int | None = None,
    tqdm_class: Callable[..., std_tqdm[tuple[_T, ...]]] = ...,
    **kwargs: Any,
) -> std_tqdm[tuple[_T, ...]]: ...
def batched(
    iterable: Iterable[_T],
    n: int,
    total: int | None = None,
    tqdm_class: Callable[..., std_tqdm[tuple[_T, ...]]] = ...,
    **kwargs: Any,
) -> std_tqdm[tuple[_T, ...]]: ...



from typing import Final, Literal

from gunicorn.config import Config
from gunicorn.http.body import Body
from gunicorn.http.unreader import Unreader

from .._types import _AddressType

MAX_UWSGI_VARS: Final = 1000

class UWSGIRequest:
    cfg: Config
    unreader: Unreader
    peer_addr: _AddressType
    remote_addr: _AddressType
    req_number: int
    method: str | None
    uri: str | None
    path: str | None
    query: str | None
    fragment: str | None
    version: tuple[int, int]
    headers: list[tuple[str, str]]
    trailers: list[tuple[str, str]]
    body: Body | None
    scheme: Literal["https", "http"]
    must_close: bool
    uwsgi_vars: dict[str, str]
    modifier1: int
    modifier2: int
    proxy_protocol_info: dict[str, str | int | None] | None  # TODO: Use TypedDict

    def __init__(self, cfg: Config, unreader: Unreader, peer_addr: _AddressType, req_number: int = 1) -> None: ...
    def force_close(self) -> None: ...
    def parse(self, unreader: Unreader) -> bytes: ...
    def set_body_reader(self) -> None: ...
    def should_close(self) -> bool: ...



import abc

class AbstractMeasurable(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def measure(self, config, now): ...

class AnonMeasurable(AbstractMeasurable):
    def __init__(self, measure_fn) -> None: ...
    def measure(self, config, now): ...



from collections.abc import Callable, Collection, Hashable, Iterable, Iterator, Mapping, MutableMapping
from decimal import Decimal
from functools import cached_property
from typing import Any, ClassVar, Generic, TypeAlias, TypeVar, overload
from typing_extensions import Self

import numpy
from networkx.classes.coreviews import AdjacencyView, AtlasView
from networkx.classes.digraph import DiGraph
from networkx.classes.reportviews import DegreeView, DiDegreeView, EdgeView, NodeView, OutEdgeView

_DataBound: TypeAlias = Mapping[str, Any]

_Node = TypeVar("_Node", bound=Hashable)
_NodeData = TypeVar("_NodeData", bound=_DataBound, default=dict[str, Any])
_EdgeData = TypeVar("_EdgeData", bound=_DataBound, default=dict[str, Any])

_NodeWithData: TypeAlias = tuple[_Node, _NodeData]
_NodePlus: TypeAlias = _Node | _NodeWithData[_Node, _NodeData]
_Edge: TypeAlias = tuple[_Node, _Node]
_EdgeWithData: TypeAlias = tuple[_Node, _Node, _EdgeData]
_EdgePlus: TypeAlias = _Edge[_Node] | _EdgeWithData[_Node, _EdgeData]
_MapFactory: TypeAlias = Callable[[], MutableMapping[str, Any]]
_NBunch: TypeAlias = _Node | Iterable[_Node] | None
_Data: TypeAlias = (
    Graph[_Node, _NodeData, _EdgeData]
    | dict[_Node, dict[_Node, _NodeData]]
    | dict[_Node, Iterable[_Node]]
    | Iterable[_EdgePlus[_Node, _EdgeData]]
    | numpy.ndarray[Any, Any]
    # | scipy.sparse.base.spmatrix
)

__all__ = ["Graph"]

class Graph(Collection[_Node], Generic[_Node, _NodeData, _EdgeData]):
    __networkx_backend__: ClassVar[str]
    node_dict_factory: ClassVar[_MapFactory]
    node_attr_dict_factory: ClassVar[_MapFactory]
    adjlist_outer_dict_factory: ClassVar[_MapFactory]
    adjlist_inner_dict_factory: ClassVar[_MapFactory]
    edge_attr_dict_factory: ClassVar[_MapFactory]
    graph_attr_dict_factory: ClassVar[_MapFactory]

    graph: dict[str, Any]
    __networkx_cache__: dict[str, Any]

    def to_directed_class(self) -> type[DiGraph[_Node, _NodeData, _EdgeData]]: ...
    def to_undirected_class(self) -> type[Graph[_Node, _NodeData, _EdgeData]]: ...
    # @_dispatchable adds `backend` argument, but this decorated is unsupported constructor type here
    # and __init__() ignores this argument
    def __new__(cls, *args, backend=None, **kwargs) -> Self: ...
    def __init__(
        self, incoming_graph_data: _Data[_Node, _NodeData, _EdgeData] | None = None, **attr: Any
    ) -> None: ...  # attr: key=value pairs
    @cached_property
    def adj(self) -> AdjacencyView[_Node, _Node, _EdgeData]: ...

    # This object is a read-only dict-like structure
    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, s: str) -> None: ...

    def __iter__(self) -> Iterator[_Node]: ...
    def __contains__(self, n: object) -> bool: ...
    def __len__(self) -> int: ...
    def __getitem__(self, n: _Node) -> AtlasView[_Node, str, Any]: ...
    def add_node(self, node_for_adding: _Node, **attr: Any) -> None: ...  # attr: Set or change node attributes using key=value
    def add_nodes_from(
        self, nodes_for_adding: Iterable[_NodePlus[_Node, _NodeData]], **attr: Any
    ) -> None: ...  # attr: key=value pairs
    def remove_node(self, n: _Node) -> None: ...
    def remove_nodes_from(self, nodes: Iterable[_Node]) -> None: ...
    @cached_property
    def nodes(self) -> NodeView[_Node, _NodeData, _EdgeData]: ...
    def number_of_nodes(self) -> int: ...
    def order(self) -> int: ...
    def has_node(self, n: _Node) -> bool: ...
    # Including subtypes' possible return types for LSP
    def add_edge(self, u_of_edge: _Node, v_of_edge: _Node, **attr: Any) -> Hashable | None: ...
    # attr: Edge data (or labels or objects) can be assigned using keyword arguments
    def add_edges_from(self, ebunch_to_add: Iterable[_EdgePlus[_Node, _EdgeData]], **attr: Any) -> None: ...
    # attr: Edge data (or labels or objects) can be assigned using keyword arguments
    def add_weighted_edges_from(
        self, ebunch_to_add: Iterable[tuple[_Node, _Node, float | Decimal | None]], weight: str = "weight", **attr: Any
    ) -> None: ...
    # attr: Edge attributes to add/update for all edges.
    def remove_edge(self, u: _Node, v: _Node) -> None: ...
    def remove_edges_from(self, ebunch: Iterable[_EdgePlus[_Node, _EdgeData]]) -> None: ...

    @overload
    def update(self, edges: Graph[_Node, _NodeData, _EdgeData], nodes: None = None) -> None: ...
    @overload
    def update(
        self,
        edges: Graph[_Node, _NodeData, _EdgeData] | Iterable[_EdgePlus[_Node, _EdgeData]] | None = None,
        nodes: Iterable[_Node] | None = None,
    ) -> None: ...

    def has_edge(self, u: _Node, v: _Node) -> bool: ...
    def neighbors(self, n: _Node) -> Iterator[_Node]: ...
    @cached_property
    # Including subtypes' possible return types for LSP
    def edges(self) -> EdgeView[_Node, _NodeData, _EdgeData] | OutEdgeView[_Node, _NodeData, _EdgeData]: ...
    def get_edge_data(self, u: _Node, v: _Node, default: Any = None) -> _EdgeData: ...
    # default:  any Python object
    def adjacency(self) -> Iterator[tuple[_Node, dict[_Node, _EdgeData]]]: ...
    @cached_property
    # Including subtypes' possible return types for LSP
    def degree(self) -> DegreeView[_Node, _NodeData, _EdgeData] | DiDegreeView[_Node, _NodeData, _EdgeData]: ...
    def clear(self) -> None: ...
    def clear_edges(self) -> None: ...
    def is_multigraph(self) -> bool: ...
    def is_directed(self) -> bool: ...
    def copy(self, as_view: bool = False) -> Self: ...
    def to_directed(self, as_view: bool = False) -> DiGraph[_Node, _NodeData, _EdgeData]: ...
    def to_undirected(self, as_view: bool = False) -> Graph[_Node, _NodeData, _EdgeData]: ...
    def subgraph(self, nodes: _NBunch[_Node]) -> Self: ...
    def edge_subgraph(self, edges: Iterable[_Edge[_Node]]) -> Self: ...

    @overload
    def size(self, weight: None = None) -> int: ...
    @overload
    def size(self, weight: str) -> float: ...

    def number_of_edges(self, u: _Node | None = None, v: _Node | None = None) -> int: ...
    def nbunch_iter(self, nbunch: _NBunch[_Node] = None) -> Iterator[_Node]: ...



# At runtime, listing submodules in __all__ without them being imported is
# valid, and causes them to be included in a star import. See #6523
import sys

__all__ = ["dom", "parsers", "sax", "etree"]  # noqa: F822  # pyright: ignore[reportUnsupportedDunderAll]

if sys.version_info >= (3, 15):
    __all__ += ["is_valid_name"]  # pyright: ignore[reportUnsupportedDunderAll]
    from xml.utils import is_valid_name as is_valid_name, is_valid_text as is_valid_text
